// Copyright 2001-2003, Danger, Inc.  All Rights Reserved.
// This file is subject to the Danger, Inc. Sample Code License,
// which is provided in the file SAMPLE_CODE_LICENSE.
// Copies are also available from http://developer.danger.com/

package com.danger.rocks;

import danger.app.*;
import danger.ui.*;
import danger.audio.*;
import danger.system.*;
import danger.util.*;

public class HeroObject
{
	public HeroObject(Rect bounds)
	{
		int	x, y;

		mHeroImage = new HeroImage();

		mDrawMode = drawmode.BLINK;
		mModeTimer = 20;
		x = (bounds.getWidth() / 2) + bounds.left;
		y = (bounds.getHeight() / 2) + bounds.top;
		mStartX = x;
		mStartY = y;
		mMotion = new MotionObject(x, y, 0, 0, bounds, 
									HeroImage.TOTAL_HERO_FRAMES,
									0,
									mHeroImage.getSizeX(),
									mHeroImage.getSizeY());
	}

	public void newLife()
	{
		mDamage = 0;
		Hardware.setLEDHSV(0, 0, 0);
		mDead = false;
		mShield = 0;
	}

	public void kill()
	{
		mDead = true;
	}

	public void reset()
	{
		mDrawMode = drawmode.INVISIBLE;
		mModeTimer = 20;
		mLEDTimer = 1;
		mMotion.setAngle(0);
		mMotion.setSpeed(0);
		mMotion.setX(mStartX);
		mMotion.setY(mStartY);
	}

	public	void draw(Pen inPen)
	{
		inPen.setColor(Color.BLACK);

		if (mHeroImage != null)
		{
			boolean draw = true;
			Rect r = getRect();

			switch (mDrawMode)
			{
				case drawmode.NORMAL:
					draw = true;
					break;
				case drawmode.DEAD:
					// draw game over
			//		inPen.DrawText(mMotion.GetX() - 20, mMotion.GetY(), RockStrings.kGameOverMan);
					Hardware.setLEDHSV(0, 0, 0);
					draw = false;
					break;
				case drawmode.INVISIBLE:
					draw = false;
					break;
				case drawmode.BLINK:
					if ((mModeTimer & 2) == 0)
					{
						draw = false;
					}
					break;
				case drawmode.SPIN_CRASH:
					break;
			}
			
			if (draw == true)
			{
				mHeroImage.draw(inPen,
							mMotion.getX(),
							mMotion.getY(),
							mMotion.getCurrentFrame());
			}

 			if (mShield > 0)
 			{
 				r.inset(-5);
 				
 				if (Screen.isColor())
 				{
 					int rc = (mFrameCount << 4) & 0xff;
 					int gc = (~((mFrameCount << 2) & 0xff)) + 255;
 					int bc = (mFrameCount << 5) & 0xff;
					inPen.setColor(Color.pack(rc, gc, bc));
					inPen.drawOval(r);
					inPen.setColor(Color.BLACK);
 				}
 				else
 				{
					inPen.drawOval(r);
				}
				mShield--;
  			}

			if (mLEDTimer > 0)
			{
				mLEDTimer--;
				if (mLEDTimer < (DAMAGE_LED_TIMER/2))
				{
					setDamageLED();
				}
			}
			else
			{
				mLEDTimer = DAMAGE_LED_TIMER;
				Hardware.setLEDHSV(0, 0, 0);
			}
			if (mSpeakTimer > 0)
			{
				mSpeakTimer--;
				if (mSpeakMsg != -1)
				{
					inPen.drawText(mMotion.getX() - SPEAK_OFFSET_X, mMotion.getY() - SPEAK_OFFSET_Y,
																	RockStrings.sHeroSpeak[mSpeakMsg]);
				}
			}

			if (rocks.mDEBUG == true)
			{			
				if (r.left > r.right)
				{
					int a = r.left;
					r.left = r.right;
					r.right = a;
				}
				if (r.bottom < r.top)
				{
					int a = r.bottom;
					r.bottom = r.top;
					r.top = a;
				}
				inPen.drawRect(r);
			}
		}
	}

	public Rect getRect()
	{
		mBoundRect = mHeroImage.getRectForHero(mMotion.getCurrentFrame());
		mBoundRect.inset(3);
		mBoundRect.translate(mMotion.getX(), mMotion.getY());
		return mBoundRect;
	}

	public Rect getImageRect()
	{
		mImageRect = mHeroImage.getRectForHero(mMotion.getCurrentFrame());
		mImageRect.inset(-5);
		mImageRect.translate(mMotion.getX(), mMotion.getY());
		return mImageRect;
	}

	public boolean isDead()
	{
		if (mDrawMode == drawmode.DEAD)
		{
			return true;
		}
		return false;
	}
	public boolean isActive()
	{
		if (mDrawMode == drawmode.NORMAL)
		{
			return true;
		}
		return false;
	}

	private static final int YELLOW[] = {40, 240, 120};
	private static final int RED[] = {0, 240, 120};
	private static final int GREEN[] = {80, 240, 120};

	private void setDamageLED()
	{
		switch (mDamage)
		{
			default:
				Hardware.setLEDHSV(0, 0, 0);
				break;
			case 25:
				Hardware.setLEDHSV(GREEN[0], GREEN[1], GREEN[2]);
				break;
			case 50:
				Hardware.setLEDHSV(YELLOW[0], YELLOW[1], YELLOW[2]);
				break;
			case 75:
				Hardware.setLEDHSV(RED[0], RED[1], RED[2]);
				break;
		}
	}

	public boolean sustainDamage()
	{
		mDamage += 25;
		if (mDamage > 99)
		{
			return false;
		}
		mShield = 6;
		return true;
	}

	public boolean hit()
	{
		boolean hit = false;
		if (mDrawMode == drawmode.NORMAL)
		{	// only hurt if we're normal
			mDrawMode = drawmode.SPIN_CRASH;
			mModeTimer = 30;
			hit = true;
			mHeroImage.setCrash(true);

			mSpeakTimer = 28;
			mSpeakMsg++;
			if (RockStrings.sHeroSpeak[mSpeakMsg] == null)
			{
				mSpeakMsg = 0;
			}
		}
		return hit;
	}
	
	private boolean safeMode()
	{
		if (mDrawMode != drawmode.NORMAL)
		{
			if (mModeTimer < 3)
			{
				mModeTimer = 3;
			}
			return false;
		}
		return true;
	}

	public boolean collidesWith(MissileObject missile)
	{
		Rect rect = getRect();
		int x = missile.mMotion.getX();
		int y = missile.mMotion.getY();

		if (missile.mOrigin != this)
		{
			return rect.contains(x, y);
		}
		return false;
	}

	public boolean collidesWith(RockObject rock)
	{
		Rect rockrect = rock.getRect();

		Rect me = getRect();
		me.inset(5);

		boolean hit = me.intersects(rockrect);
		if (rock.isCrashed() == true)
		{
			hit = false;
		}
		if (hit)
		{
			return safeMode();
		}
		return false;
	}


	public	void update(RockWindow w)
	{
		mFrameCount++;

		// invalidate old position
		w.InvalidateThisRect(getImageRect());
		mMotion.Update();
		// invalidate new position
		w.InvalidateThisRect(getImageRect());

		switch (mDrawMode)
		{
			case drawmode.NORMAL:
				break;
			case drawmode.INVISIBLE:
				mModeTimer--;
				if (mModeTimer == 0)
				{
					mDrawMode = drawmode.BLINK;
					mModeTimer = 20;
					mHeroImage.setCrash(false);
				}
				break;
			case drawmode.BLINK:
				mModeTimer--;
				if (mModeTimer == 0)
				{
					mDrawMode = drawmode.NORMAL;
				}
				break;
			case drawmode.SPIN_CRASH:
				mModeTimer--;
				if (mModeTimer != 0)
				{
					mMotion.setAngle(mMotion.getAngle() + 10);
				}
				else
				{
					if (mDead == false)
					{
						reset();
					}
					else
					{
						mDrawMode = drawmode.DEAD;
					}
				}
				break;
		}
		if (mThrustOn)
		{
			if ( (mFrameCount & 1) == 0)
			{
				mMotion.addSpeed(FIXED_TWO_THRIDS);
			}
		}
	}

	public void rotateLeft()
	{
		mMotion.addAngle(-HeroImage.HERO_ANGLE_PER_FRAME);
	}
	
	public void rotateRight()
	{
		// rotate right
		mMotion.addAngle(HeroImage.HERO_ANGLE_PER_FRAME);
	}

	private static final int FIXED_TWO_THRIDS	= Fixed.fromReal(0, 2, 3);

	public void startThrust()
	{
		mHeroImage.setThrust(true);
		RockAudio.startThrust();
		mThrustOn = true;
	}

	public void stopThrust()
	{
		mHeroImage.setThrust(false);
		RockAudio.stopThrust();
		mThrustOn = false;
	}

	interface drawmode
	{
		public static final int NORMAL			= 0;
		public static final int SPIN_CRASH		= 1;
		public static final int BLINK			= 2;
		public static final int INVISIBLE		= 3;
		public static final int DEAD			= 4;
	}

	private int			mSpeakMsg = -1;
	private int			mSpeakTimer = 0;
	private static int	SPEAK_OFFSET_X = 11;
	private static int	SPEAK_OFFSET_Y = 4;
	private static int	DAMAGE_LED_TIMER	= 30;

	// data
	boolean				mThrustOn;
	int					mDamage;
	int					mShield;
	int					mFrameCount;
	boolean				mDead;
	int					mDrawMode;
	int					mModeTimer = 0;
	int					mLEDTimer = 1;
	int					mStartX;
	int					mStartY;
	HeroImage			mHeroImage = null;
	MotionObject		mMotion = null;
	Rect				mImageRect = new Rect();
	Rect				mBoundRect = new Rect();
}


