// Copyright 2001-2003, Danger, Inc.  All Rights Reserved.
// This file is subject to the Danger, Inc. Sample Code License,
// which is provided in the file SAMPLE_CODE_LICENSE.
// Copies are also available from http://developer.danger.com/

package com.danger.rocks;

import danger.ui.*;
import danger.app.*;
import danger.audio.*;
import danger.util.LinkedList;

public class RockObject
{
	public	RockObject(int x, int y, int angle, int fspeed, Rect bounds, RockImage rock)
	{

		mMotion = new MotionObject(x, y, angle, fspeed, bounds, 
									RockImage.TOTAL_ROCK_FRAMES,
									RockImage.ROCK_FRAME_RATE,
									rock.getSizeX(),
									rock.getSizeY());
		mRockImage = null;
		if (mMotion != null)
		{
			mRockImage = rock;
			setCrash(false);
			mMotion.setCurrentFrame(mMotion.getFrameFromAngle());
		}
	}

	public	void draw(Pen inPen)
	{
		if (mRockImage != null)
		{
			int x = mMotion.getX();
			int y = mMotion.getY();
			int f = mMotion.getCurrentFrame();

			if (mCrashFrame < TOTAL_CRASH_FRAMES/2)
			{
				mRockImage.draw(inPen, x, y, f, false);
			}
			if (mCrashed)
			{
				mRockImage.draw(inPen, x, y, f, true);

				if (mCrashFrame < TOTAL_CRASH_FRAMES)
				{
					mCrashFrame++;
				}
				else
				{
					// we hit the end, so delete from list
					if (mRockLink != null)
					{
						mRockLink.remove(this);
						mRockLink = null;
					}
				}
			}

			if (rocks.mDEBUG == true)
			{
				Rect r = getRect();
			
				if (r.left > r.right)
				{
					int a = r.left;
					r.left = r.right;
					r.right = a;
				}
				if (r.bottom < r.top)
				{
					int a = r.bottom;
					r.bottom = r.top;
					r.top = a;
				}
				inPen.drawRect(r);
			}
		}
	}

	public Rect getRect()
	{
		mRockImage.getRectForFrame(mMotion.getCurrentFrame(), mImageRect);
		mImageRect.translate(mMotion.getX(), mMotion.getY());
		return mImageRect;
	}

	public void hit(LinkedList list)
	{
		mRockLink = list;
		setCrash(true);
	}

	public boolean collidesWith(MissileObject missile)
	{
		boolean hit = false;

		if (mCrashed == false)
		{
			Rect rect = getRect();
			hit = rect.contains(missile.mMotion.getX(), missile.mMotion.getY());
		}
		return hit;
	}

	public void setCrash(boolean crash)
	{
		mCrashFrame = 0;
		mCrashed = crash;
	}

	public boolean isCrashed()
	{
		return mCrashed;
	}

	public	void update(RockWindow w)
	{
		// invalidate old position
		w.InvalidateThisRect(getRect());
		mMotion.Update();
		// invalidate new position
		w.InvalidateThisRect(getRect());
	}

	// data
	RockImage				mRockImage = null;
	MotionObject			mMotion = null;
	Rect					mImageRect = new Rect();
	LinkedList				mRockLink;
	boolean					mCrashed;
	int						mCrashFrame;

	static final int		TOTAL_CRASH_FRAMES	= 4;
}

