// Copyright 2001-2003, Danger, Inc.  All Rights Reserved.
// This file is subject to the Danger, Inc. Sample Code License,
// which is provided in the file SAMPLE_CODE_LICENSE.
// Copies are also available from http://developer.danger.com/

/* Asteroids! */
package com.danger.rocks;

import danger.app.*;
import danger.ui.*;
import danger.util.format.StringFormat;
import danger.audio.*;
import danger.util.*;

class RockWindow extends ScreenWindow
{
	//*	--------------------	RockWindow
	public
	RockWindow()
	{
  		BuildActionMenu();

		mBounds = getBounds();

		mFont = Font.findBoldSystemFont();
		mFontOutline = Font.findBoldOutlineSystemFont();
		mGameState = INTRO_SCREEN;
		mPlayfieldID = 0;
		mPlayfieldOsc = 0;
		mMenuShown = false;
	}

	void NewPlayfield()
	{
		if (rocks.sIsColor == true)
		{
			mPlayfieldOsc++;
			if (mPlayfieldOsc > 2)
			{
				mPlayfieldOsc = 0;
				mPlayfieldID++;
				if (sPlayfields[mPlayfieldID] == 0)
				{
					mPlayfieldID = 0;
				}
			}
			loadPlayfield();
		}
	}

	private void loadPlayfield()
	{
		mPlayfield = null;
//		mPlayfield = rocks.mMe.getBitmap(sPlayfields[mPlayfieldID]);
		mPlayfield = getJpegImage(sPlayfields[mPlayfieldID]);
	}
	
	private Bitmap getJpegImage(int resourceID)
	{
		Resource data = rocks.mMe.getResource(2000, resourceID);
		Bitmap image = null;

	//	System.out.println("trying to decode image " + resourceID);
		if (data != null)
		{
			int size = data.getSize();
			byte [] bytes = new byte[size];
			
			if (bytes != null)
			{
				data.getBytes(bytes, 0, size);
				try
				{
					
					image = Bitmap.newFromJPEGData(bytes);
				}
				catch (Exception x)
				{
					System.out.println("failed to decode jpeg");
				}
				if (image != null)
				{
					// convert to screen space, because decoding jpeg during frame kills our speed
					image = Bitmap.convertToScreenColorSpace(image);
				}
			}
		}
		else
		{
			System.out.println("No resource " + resourceID);
		}
		return image;
	}

	// load bitmaps
	void LoadBitmaps()
	{
		int count;

		if (mBitmapsReady == false)
		{
			if (rocks.sIsColor == true)
			{
				loadPlayfield();
			}
			mIntroBitmap = rocks.mMe.getBitmap(Resources.ID_IntroPage);
			mAnnounce = rocks.mMe.getBitmap(Resources.ID_announce);
			
			mCorners[0] = rocks.mMe.getBitmap(Resources.ID_display_corner_f01);
			mCorners[1] = rocks.mMe.getBitmap(Resources.ID_display_corner_f02);
			mCorners[2] = rocks.mMe.getBitmap(Resources.ID_display_corner_f03);
			mCorners[3] = rocks.mMe.getBitmap(Resources.ID_display_corner_f04);

			mLivesBitmap[0] = rocks.mMe.getBitmap(Resources.ID_lives_f01);
			mLivesBitmap[1] = rocks.mMe.getBitmap(Resources.ID_lives_f02);
			mLivesBitmap[2] = rocks.mMe.getBitmap(Resources.ID_lives_f03);
			mLivesBitmap[3] = rocks.mMe.getBitmap(Resources.ID_lives_f04);
			mLivesBitmap[4] = rocks.mMe.getBitmap(Resources.ID_lives_f05);
			mLivesBitmap[5] = rocks.mMe.getBitmap(Resources.ID_lives_f06);

			mHealthBitmap[0] = rocks.mMe.getBitmap(Resources.ID_health_f01);
			mHealthBitmap[1] = rocks.mMe.getBitmap(Resources.ID_health_f02);
			mHealthBitmap[2] = rocks.mMe.getBitmap(Resources.ID_health_f03);
			mHealthBitmap[3] = rocks.mMe.getBitmap(Resources.ID_health_f04);
			mBitmapsReady = true;
		}
	}
	
	void FreeBitmaps()
	{
		int count;

		mIntroBitmap = null;
		mPlayfield = null;
		mAnnounce = null;

		for (count = 0; count < 4;  count++)
		{
			mCorners[count] = null;
		}
		for (count = 0; count < 6; count++)
		{
			mLivesBitmap[count] = null;
		}
		for (count = 0; count < 4; count++)
		{
			mHealthBitmap[count] = null;
		}
		mBitmapsReady = false;
	}

	final void BuildWorld()
	{
		int rockCount;

		mObjectsReady = false;

		RockImage.setupRockImages();

		int ex = mBounds.getWidth();
		int ey = mBounds.getHeight();
		rockCount = INITIAL_ROCKS + (mLevel / 3);
		if (rockCount > MAX_ROCKS)
		{
			rockCount = MAX_ROCKS;
		}
		for (int count = 0; count < rockCount; count++)
		{
			int angle = rocks.mRandom.nextInt(360);
			if ( ((angle / 90) * 90) == angle)
			{
				angle = angle + 3;	// don't allow 90 degrees
			}

			mRocksList.add (new RockObject(
								ex + rocks.mRandom.nextInt(21) - 10,
								ey + rocks.mRandom.nextInt(21) - 10,
								angle, 
								Fixed.fromInt(1) + rocks.mRandom.nextInt(0x7001) - 0x1000,
								mBounds,
								RockImage.getRockImage(RockImage.ROCK_LARGE)));
		}
		mObjectsReady = true;
	}

	void ClearMissiles()
	{
		for (int count = 0; count < MAX_MISSILES; count++)
		{
			mMissiles[count] = null;
		}
		mCurrentMissle = 0;
	}

	protected void
	PreEnterHook()
	{
		mMenuShown = true;
	}

	protected void
	PreExitHook()
	{
		mMenuShown = false;
	}

	// Called right before the menu gets shown
	protected void
	        aboutToShowMenu()
	{
		if (mMenuShown == false)
		{
			if (mGameState != INTRO_SCREEN)
			{
				rocks.pauseGame();
			}
		}
	}

	// Called right before the menu gets hidden
	protected void
	        aboutToHideMenu()
	{
		if (mMenuShown == true)
		{
			if (mGameState != INTRO_SCREEN)
			{
				rocks.resumeGame();
			}
		}
	}

  	final void
  	BuildActionMenu()
  	{
		adjustActionMenuState();
  	}

	public void
            adjustActionMenuState()
	{
  		Menu menu = getActionMenu();
  		menu.removeAllItems();

		if (mGameState != INTRO_SCREEN)
		{
			menu.addItem(RockStrings.kStr_ResumeGame, rocks.kCmd_ResumeGame, 0, null, rocks.mMe);

			if (RockAudio.isSoundtrackEnabled())
			{
				menu.addItem(RockStrings.kStr_SoundtrackOff, rocks.kCmd_SoundtrackOff, 0, null, rocks.mMe);
			}
			else
			{
				menu.addItem(RockStrings.kStr_SoundtrackOn, rocks.kCmd_SoundtrackOn, 0, null, rocks.mMe);
			}
		}
  		menu.addItem(RockStrings.kStr_NewGame, rocks.kCmd_NewGame, 0, null, rocks.mMe);
		menu.addDivider();
		if (mGameState != INTRO_SCREEN)
		{
	  		menu.addItem(RockStrings.kStr_ExitGame, rocks.kCmd_ExitGame, 0, null, rocks.mMe);
		}
		else
		{
	  		menu.addItem(RockStrings.kStr_Quit, rocks.kCmd_Quit, 0, null, rocks.mMe);
		}
	}

	//*	--------------------	GetWindowTitle

	public String
	GetWindowTitle()
	{
		return (RockStrings.kAppTitle);
	}

	//*	--------------------	Paint

	public void Update()
	{
		mHero.update(this);
		if (mAntagonist != null)
		{
			mAntagonist.update(this);
		}
	}

	public void InvalidateThisRect(Rect r)
	{
	//	Invalidate(r);
	}

	private void DrawInformationText(Pen inPen)
	{
		int size;

		inPen.setFont(mFontOutline);
		// draw score
		inPen.drawText(10, 15, StringFormat.withFormat(RockStrings.kScore, String.valueOf(mScore)));

		switch (mGameState)
		{
			case INTRO_SCREEN:
				inPen.drawText(10, 15 + 9, StringFormat.withFormat(RockStrings.kHighScoreText, String.valueOf(rocks.mDS.GetHighScore())));
				break;
			case IN_GAME_SCREEN:
				if (mHero.isActive() == true)
				{
					inPen.drawBitmap(145, 8, mHealthBitmap[mHero.mDamage / 25]);
				}
				if (mLives != 0)
				{
					inPen.drawBitmap(188, 8, mLivesBitmap[mLives - 1]);
				}

			/*	if (rocks.mDEBUG == true)
				{
					inPen.DrawText(160, 35, "a " + mTimeBetweenShots + " " + mLastShot);
				}
			*/
				break;
		}
	}

	public void show()
	{
		LoadBitmaps();
		super.show();
	}
	
	public void hide()
	{
		super.hide();
		FreeBitmaps();
	}

	public void resume()
	{
	}
	
	public void pause()
	{
		mHero.stopThrust();
	}
	public void paint(Pen inPen)
	{
		int	y, textline, x;

		if (mBitmapsReady == true)
		{
			inPen.setColor(Color.BLACK);
			inPen.setTextOutlineColor(mStyle.getColor(Style.SPLASH_SCREEN_TEXT_OUTLINE_COLOR));

			switch (mGameState)
			{
				case INTRO_SCREEN:
					String [] message;

					inPen.drawBitmap(0, 0, mIntroBitmap);
	
					inPen.setFont(mFontOutline);
					
					if (mShowCredits == false)
					{
						message = RockStrings.sInstructions;
					}
					else
					{
						message = RockStrings.sCredits;
					}

					y = 23;
					y += 10 * 2;
					for (textline = 0; message[textline] != null; textline++)
					{
						inPen.drawText (10, y, message[textline]);
						y += 10;
					}
					
					DrawInformationText(inPen);
					break;
	
				case BUILDING_SCREEN:
					DrawInformationText(inPen);
					break;
				case IN_GAME_SCREEN:
					inPen.setFont(mFont);
					if ((rocks.sIsColor == true) && (mPlayfield != null))
					{
						inPen.drawBitmap(0, 0, mPlayfield);
					}
					else
					{
						clear(inPen);
					}			
					if (mObjectsReady)
					{
						int count;
						ListIterator iterator;
						RockObject rock;
	
						iterator = mRocksList.listIterator(mRockListIterator);
						for (rock = (RockObject) iterator.next(); rock != null ; rock = (RockObject) iterator.next())
						{
							rock.draw(inPen);
						}
						for (count = 0; count < MAX_MISSILES; count++)
						{
							if (mMissiles[count] != null)
							{
								if (mMissiles[count].mOrigin == mHero)
								{
									mMissiles[count].draw(inPen, 1);
								}
								else
								{
									mMissiles[count].draw(inPen, 0);
								}
							}
						}
						
						mHero.draw(inPen);
	
						if (mAntagonist != null)
						{
							mAntagonist.draw(inPen);
						}
					}
	
					DrawInformationText(inPen);
					if (mAnnouncer > 0)
					{
						inPen.setFont(mFont);
						inPen.drawBitmap(mAnnounceX, 0, mAnnounce);
						inPen.drawText(mAnnounceX + 55, 18, mAnnounceString);
						
						if (mAnnouncer < ANNOUNCE_DELTA_MID)
						{
							if (mAnnounceWait > 0)
							{
								mAnnounceWait--;
							}
							else
							{
								mAnnouncer--;
								mAnnounceX += 32;
							}
						}
						else
						{
							mAnnouncer--;
							mAnnounceX += 32;
						}
					}
					break;
			}
			if (rocks.isGamePaused() == true)
			{
				int w = mBounds.getWidth();
				inPen.setFont(mFontOutline);
				y = 55;
				x = (w - mFontOutline.getWidth(RockStrings.kPaused)) / 2;
				inPen.drawText(x, y, RockStrings.kPaused);

				x = (w - mFontOutline.getWidth(RockStrings.kPressBack)) / 2;
				y += 9;
				inPen.drawText(x, y, RockStrings.kPressBack);
			}
	
			// draw corners
			inPen.drawBitmap(0, 0, mCorners[0]);
			inPen.drawBitmap(240-9, 0, mCorners[1]);
			inPen.drawBitmap(240-9, 136-9, mCorners[2]);
			inPen.drawBitmap(0, 136-9, mCorners[3]);
		}
	}

	void SetScore(int score)
	{
		mScore = score;
		mExtraLife = rocks.EXTRA_LIFE_AT - (mScore % rocks.EXTRA_LIFE_AT);
		mAntogonistAltScore = rocks.ALT_AT - (mScore % rocks.ALT_AT);
	}

	public void AddToScore(int amount)
	{
		mScore += amount;

		if (rocks.mDS.GetHighScore() < mScore)
		{
			rocks.mDS.SetHighScore(mScore);
		}

		// Add a life if we hit multiple of 30k

		mExtraLife -= amount;
		if (mExtraLife < 0)
		{
			mExtraLife = rocks.EXTRA_LIFE_AT;
			mLives++;
			if (mLives > rocks.MAX_LIVES)
			{
				mLives = rocks.MAX_LIVES;
			}
			else
			{
				RockAudio.extraLife();
			}
		}

		mAntogonistAltScore -= amount;
		if (mAntogonistAltScore < 0)
		{
			mAntogonistAltScore = rocks.ALT_AT;
			mAltLives = 2;
		}
	}

	public void StartAnnouncer(String announceMe)
	{
		mAnnouncer = ANNOUNCE_DELTA;
		mAnnounceX = -190;
		mAnnounceWait = ANNOUNCE_WAIT_TIME;
		mAnnounceString = announceMe;
	}

	public void StartAnnouncer()
	{
		mAnnouncer = ANNOUNCE_DELTA;
		mAnnounceX = -190;
		mAnnounceWait = ANNOUNCE_WAIT_TIME;
		mAnnounceString = RockStrings.sComplete[mAnnounceCount];

		mAnnounceCount++;
		if (RockStrings.sComplete[mAnnounceCount] == null)
		{
			mAnnounceCount = 0;
		}
	}
	public void StopAnnouncer()
	{
		mAnnouncer = 0;
	}
	public boolean IsAntagonistDead()
	{
		if (mAntagonist == null)
		{
			return true;
		}
		return mAntagonist.isDead();
	}
	public void NewAntagonist()
	{
		if (mAntagonist == null)
		{
			mAntagonist = new AntagonistObject(mBounds);
		}

		if (mAltLives > 0)
		{
			mAltLives--;
			mAntagonist.newLife(true);
		}
		else
		{
			mAntagonist.newLife(false);
		}
		mAntagonistLastTimeOnScreen = WAIT_TIME_ANTAGONIST;
	}

	public void HitHero()
	{
		if (mHero.hit() == true)
		{
			RockAudio.hitHero();
			if (mLives == 0)
			{
				mHero.kill();
				mResetGameTimer = 60;
				StartAnnouncer(RockStrings.kGameOverMan);
			}
			else
			{
				mHero.newLife();
				mLives--;
			}
		}
	}

	void AntagonistLaunchMissile()
	{
		if (mAntagonist != null)
		{
			if (mAntagonist.isActive())
			{
				RockAudio.fireMissle();
				if (mMissiles[mCurrentMissle] == null)
				{
					mMissiles[mCurrentMissle] = new MissileObject(mBounds);
				}
			
				// now reset the missile location, angle, and offset by ship size
				// so that it comes out of something graphical. :)
				int cframe = mAntagonist.mMotion.getCurrentFrame();
				int ox = 12;
				int oy = 12;
				mMissiles[mCurrentMissle].create(mAntagonist, mAntagonist.mMotion.getX() + ox,
												mAntagonist.mMotion.getY() + oy);


		//		mMissiles[mCurrentMissle].SetAngle(mAntagonist.mMotion.GetAngleFromFrame());
				mMissiles[mCurrentMissle].setAngle(rocks.mRandom.nextInt(360));

				mMissiles[mCurrentMissle].mMotion.setSpeedFromVector(mAntagonist.mMotion);
				mMissiles[mCurrentMissle].mMotion.addSpeed(Fixed.fromInt(7));
				mCurrentMissle = (mCurrentMissle + 1) % MAX_HERO_MISSILES;

			}
		}
	}

	void LaunchMissile()
	{
		if (mHero.isActive())
		{
			RockAudio.fireMissle();
			if (mMissiles[mCurrentMissle] == null)
			{
				mMissiles[mCurrentMissle] = new MissileObject(mBounds);
			}
		
			// now reset the missile location, angle, and offset by ship size
			// so that it comes out of something graphical. :)
			int cframe = mHero.mMotion.getCurrentFrame();
			int ox = HeroImage.mHeroMissileOffsetX[cframe];
			int oy = HeroImage.mHeroMissileOffsetY[cframe];
			mMissiles[mCurrentMissle].create(mHero, mHero.mMotion.getX() + ox,
											mHero.mMotion.getY() + oy);
			mMissiles[mCurrentMissle].setAngle(mHero.mMotion.getAngleFromFrame());

		
			mMissiles[mCurrentMissle].mMotion.setSpeedFromVector(mHero.mMotion);
			mMissiles[mCurrentMissle].mMotion.addSpeed(Fixed.fromInt(7));
			mCurrentMissle = (mCurrentMissle + 1) % MAX_HERO_MISSILES;

			mLastShot = rocks.mGameFrame;
		}
	}

	public boolean
            eventWidgetHeld(int inWhichWidget, Event event)
	{
	boolean	consumed = false;
		switch (inWhichWidget)
		{
			case Event.DEVICE_WHEEL_BUTTON:
				mAutoFire = true;
				mAutoFireTimer = 0;
				consumed = true;
				break;
		}
		if (consumed == false)
		{
			return super.eventWidgetHeld(inWhichWidget, event);
		}
		return true;
	}

	public boolean
            eventWidgetUp(int inWhichWidget, Event event)
	{
	boolean	consumed = false;

		switch (mGameState)
		{
			case IN_GAME_SCREEN:
				if (mObjectsReady && rocks.isGamePaused() == false)
				{
					if (mHero.isDead() == false)
					{
						switch (inWhichWidget)
						{
							case Event.DEVICE_WHEEL:		// wheel scroll up
								if (mWheelRotationDirection > 0)
								{
									mHero.rotateRight();
								}
								else
								{
									mHero.rotateLeft();
								}
								consumed = true;
								break;

							case Event.DEVICE_BUTTON_BACK:
								rocks.pauseGame();
								consumed = true;
								break;

							case Event.DEVICE_WHEEL_BUTTON:
								mAutoFire = false;
								mAutoFireTimer = 0;
								consumed = true;
								break;

							case Event.DEVICE_BUTTON_JUMP:
								mHero.stopThrust();
								consumed = true;
								break;
						}
					}
				}
				else
				{
					switch (inWhichWidget)
					{
						case Event.DEVICE_BUTTON_BACK:
						case Hiptop2Events.DEVICE_BUTTON_CANCEL:
						case Event.DEVICE_BUTTON_JUMP:
							rocks.pauseGame();
							rocks.mMe.returnToLauncher();
							consumed = true;
							break;
					}
				}
				break;
			case INTRO_SCREEN:
				switch (inWhichWidget)
				{
					case Hiptop2Events.DEVICE_BUTTON_CANCEL:
					case Event.DEVICE_BUTTON_BACK:
						rocks.mMe.returnToLauncher();
						consumed = true;
						break;
					case Event.DEVICE_WHEEL_BUTTON:
						Meta.play(Meta.BEEP_COMMAND_ACCEPTED);
						rocks.mMe.newGame();
						consumed = true;
						break;
				}
				break;
		}
		if (consumed == false)
		{
			return super.eventWidgetUp(inWhichWidget, event);
		}
		return true;
	}

	//*	--------------------	EventDeviceWidgetDown
	public boolean
            eventWidgetDown(int inWhichWidget, Event event)
	{
	boolean	consumed = false;

		switch (mGameState)
		{
			case IN_GAME_SCREEN:
				if (mObjectsReady && rocks.isGamePaused() == false)
				{
					if (mHero.isDead() == false)
					{
						switch (inWhichWidget)
						{
							case Event.DEVICE_WHEEL_BUTTON:
								LaunchMissile();
								consumed = true;
								break;

							case Event.DEVICE_BUTTON_JUMP:
								mHero.startThrust();
								consumed = true;
								break;
								
							case Event.DEVICE_WHEEL:			// wheel scroll down
								if (mWheelRotationDirection > 0)
								{
									mHero.rotateLeft();
								}
								else
								{
									mHero.rotateRight();
								}
								consumed = true;
								break;								
						}
					}
				}
				else
				{
					switch (inWhichWidget)
					{
						case Event.DEVICE_BUTTON_JUMP:
							rocks.pauseGame();
							rocks.mMe.returnToLauncher();
							consumed = true;
							break;
					}
				}
				break;
		}

		if (consumed == false)
		{
			return super.eventWidgetDown(inWhichWidget, event);
		}
		return true;
	}


	private void DebugKeys(char inKey)
	{
		if (rocks.mDEBUG == true)
		{
			switch (inKey)
			{
				case 'l':
					SetScore(rocks.EXTRA_LIFE_AT - 200);
					break;
				case 'k':
					HitHero();
					break;
				case 'x':
					SetScore(rocks.ALT_AT - 200);
					break;
				case 's':
					mHero.mShield = 1000;
					break;
				case 'f':
					StartAnnouncer();
					break;
				case 'n':
					NewPlayfield();
					break;
				case 'g':
					rocks.mGOD = (rocks.mGOD == false) ? true : false;
					Meta.play(Meta.BEEP_COMMAND_ACCEPTED);
					break;
				case '1':
					mHero.mHeroImage.mHeroMissileOffsetX[mHero.mMotion.getCurrentFrame()]--;
					break;
				case '2':
					mHero.mHeroImage.mHeroMissileOffsetX[mHero.mMotion.getCurrentFrame()]++;
					break;
				case '3':
					mHero.mHeroImage.mHeroMissileOffsetY[mHero.mMotion.getCurrentFrame()]--;
					break;
				case '4':
					mHero.mHeroImage.mHeroMissileOffsetY[mHero.mMotion.getCurrentFrame()]++;
					break;
				case '5':
					for (int count = 0; count < HeroImage.TOTAL_HERO_FRAMES; count++)
					{
						DEBUG.p(count + " " + mHero.mHeroImage.mHeroMissileOffsetX[count] + " " +
														mHero.mHeroImage.mHeroMissileOffsetY[count]);
					}
					break;
			}
		}
		switch (inKey)
		{
			case 'd':
				if (rocks.mDEBUG == true)
				{
					rocks.mDEBUG = false;
				}
				else
				{
					rocks.mDEBUG = true;
				}
				Meta.play(Meta.BEEP_COMMAND_ACCEPTED);
				break;
		}
	}


	public boolean
            eventKeyDown(char inKey, Event event)
	{
		RockAudio.handleKeys(inKey);
		DebugKeys(inKey);

		if (mObjectsReady && (mHero.isDead() == false))
		{
			switch (inKey)
			{
				case 'i':
					mWheelRotationDirection *= -1;
					Meta.play(Meta.BEEP_COMMAND_ACCEPTED);
					return true;
			}
		}
		else
		{
			switch (inKey)
			{
				case 'a':
					mShowCredits = (mShowCredits) ? false : true;
					Meta.play(Meta.BEEP_COMMAND_ACCEPTED);
					rocks.mWindow.invalidate();
					return true;
			}
		}

		return super.eventKeyDown(inKey, event);
	}

	// constants
	static final int	INTRO_SCREEN		= 0;
	static final int	BUILDING_SCREEN		= 1;
	static final int	IN_GAME_SCREEN		= 2;

	static final int	INITIAL_ROCKS		= 3;
	static final int	MAX_ROCKS			= 8;
	static final int	MAX_MISSILES		= 5;		// max number of missiles in the air
	static final int	MAX_HERO_MISSILES	= 5;

	static final int	WAIT_TIME_ANTAGONIST = 50;

	static final int	ANNOUNCE_DELTA		= 15;
	static final int	ANNOUNCE_DELTA_MID	= 10;
	static final int	ANNOUNCE_WAIT_TIME	= 10;

	//*	--------------------------------	Instance Variables

	Font				mFont;
	Font				mFontOutline;
	Rect				mBounds;

	boolean				mMenuShown;

	int					mLevel;

	int					mScore;
	int					mExtraLife;
	int					mLives;
	byte []				mTextBuffer = new byte[50];

	boolean				mShowCredits;
	boolean				mBitmapsReady;
	Bitmap				mIntroBitmap;
	Bitmap				mPlayfield;
	Bitmap 				mCorners[] = {null, null, null, null};
	
	Bitmap				mAnnounce;
	int					mAnnouncer;
	int					mAnnounceX;
	int					mAnnounceWait;
	String				mAnnounceString;
	int					mAnnounceCount;

	Bitmap				mLivesBitmap[] = {null, null, null, null, null, null};
	Bitmap				mHealthBitmap[] = {null, null, null, null};

	boolean				mObjectsReady = false;
	boolean				mAutoFire;
	int					mAutoFireTimer;
	int					mAntagonistAutoFireTimer;
	int					mAntagonistLastTimeOnScreen;
	int					mAntogonistAltScore;
	int					mAltLives;

	int					mTimeBetweenShots;
	int					mLastShot;

	int					mGameState = INTRO_SCREEN;
	int					mPlayfieldID;
	int					mPlayfieldOsc;

	int					mWheelRotationDirection = 1;

	int					mResetGameTimer = 0;

	LinkedList			mRocksList;		/*RockObject*/
	ListIterator		mRockListIterator = new ListIterator();

	HeroObject			mHero = null;
	AntagonistObject	mAntagonist = null;

	int					mCurrentMissle = 0;
	MissileObject		mMissiles[] = {null, null, null, null, null, null, null, null, null, null};

	static final int	sPlayfields[] =
	{
		Resources.ID_playfield1,
		Resources.ID_playfield2,
		Resources.ID_playfield3,
		Resources.ID_playfield4,
		Resources.ID_playfield6,
		Resources.ID_playfield7,
		Resources.ID_playfield8,
		Resources.ID_playfield9,
		Resources.ID_playfield10,
		0
	};
}



class RockUtils
{
	public static int ConvertNumberToBytes(int number, byte [] result, int startOffset)
	{
		int r = 0;
		int q = 0;
		int nn = number;
		int count = startOffset;
		byte v;

		do 
		{
			r = nn % 10;
			nn = nn / 10;
			result[count++] = (byte)(48 + r);
		}
		while (nn > 0);
		result[count] = 0;

		for (nn = 0; nn < count/2; nn++)
		{
			v = result[nn];
			result[nn] = result[count-nn-1];
			result[count-nn-1] = v;
		}

		return count;
	}
}


