// Copyright 2001-2003, Danger, Inc.  All Rights Reserved.
// This file is subject to the Danger, Inc. Sample Code License,
// which is provided in the file SAMPLE_CODE_LICENSE.
// Copies are also available from http://developer.danger.com/

/* Asteroids! */


package com.danger.rocks;

import danger.app.*;
import java.util.Random;
import danger.ui.*;
import danger.util.format.StringFormat;
import danger.audio.*;
import danger.system.*;
import danger.util.*;
import java.lang.reflect.*;

public class rocks extends Application implements Resources
{
	//*	--------------------	rocks

	public
	rocks()
	{
		mMe = this;
		mRandom = new Random();
		sIsColor = Screen.isColor();

		RockStrings.setup();

		//*	Create our app's main window
		mWindow = new RockWindow();
		mWindow.setTitle(mWindow.GetWindowTitle());

		mDS = new RocksStore();

        // this is a reflection trick to keep a single version of the
        // app for all OS versions
        try
        {
            Method m = Class.forName("danger.app.ResourceDatabase").getMethod("getSplashScreen", new Class[] {int.class});
            // -1008 is AppResources.ID_SPLASH_SCREEN_RESOURCE
            mSplashScreen = (Container) m.invoke(getResources(), new Object[] {new Integer(-1008)});
		    mNumRemaining = (StaticTextBox) mSplashScreen.getDescendantWithID(ID_HIGH_SCORE);
		    mNumRemaining.setVisible(true);
        }
        catch (Exception e)
        {
            // do nothing under 1.1
        }

		updatePreviewScreen();
	}

 	//*	--------------------	getInstance

	public static final rocks getInstance()
	{
		return mMe;
	}

	public void renderSplashScreen(View inView, Pen inPen)
	{
        // if mSplashScreen is null, this is running in 1.1
        if (mSplashScreen != null)
        {
		    String text = StringFormat.withFormat(RockStrings.kHighScoreText, String.valueOf(mDS.GetHighScore()));
    
		    mNumRemaining.setText(text);
		    mSplashScreen.paint(inPen);
        }
        else
        {
		    inPen.drawBitmap(0, 0, getBitmap(AppResources.ID_SPLASH_SCREEN));
    
		    if (mDS != null)
		    {
			    inPen.setFont(Font.findBoldOutlineSystemFont());
			    inPen.setColor(inView.getStyle().getColor(Style.SPLASH_SCREEN_TEXT_COLOR));
			    inPen.setTextOutlineColor(inView.getStyle().getColor(Style.SPLASH_SCREEN_TEXT_OUTLINE_COLOR));
			    inPen.drawText(4, 32, StringFormat.withFormat(RockStrings.kHighScoreText, String.valueOf(mDS.GetHighScore())));
		    }
        }
	}



	//*	--------------------	NewGame
	public void
	        newGame()
	{
		mWindow.mObjectsReady = false;
		mWindow.mGameState = mWindow.BUILDING_SCREEN;
		mWindow.mShowCredits = false;
		mWindow.StopAnnouncer();
		mWindow.mPlayfieldID = 0;
		mWindow.mPlayfieldOsc = 0;
		mWindow.NewPlayfield();
		mWindow.ClearMissiles();

		mWindow.mLives = STARTING_LIVES;
		mWindow.mAltLives = 0;
		mWindow.SetScore(0);

		mWindow.invalidate();
		mWindow.mTimeBetweenShots = 0;
		mWindow.mLastShot = mGameFrame;

		mWindow.mRocksList = null;		// kill previous list of rocks
		mWindow.mRocksList = new LinkedList();

		if (mWindow.mHero != null)
		{
			mWindow.mHero.reset();
		}
		else
		{
			mWindow.mHero = new HeroObject(mWindow.mBounds);
		}
		mWindow.mHero.newLife();

		if (mWindow.mAntagonist != null)
		{
			mWindow.mAntagonist.kill();
		}
		mWindow.mLevel = 0;
		mWindow.BuildWorld();
		mWindow.mGameState = mWindow.IN_GAME_SCREEN;
		mWindow.invalidate();
		RockAudio.resetLevel();
	}

	public void restartGame()
	{
		Hardware.setLEDHSV(0, 0, 0);
		mWindow.mGameState = mWindow.INTRO_SCREEN;
		mWindow.mObjectsReady = false;
	}

	private void splitRock(MotionObject sourceObject, RockObject rockToSplit, int vectorDivisor, boolean addScore)
	{
		// we want the rocks to move faster than the source object

		int vectorSpeed = Fixed.divide(sourceObject.getSpeed(), vectorDivisor);
		int vectorAngle = sourceObject.getAngle();

		if (vectorSpeed < 0x2000)
		{
			vectorSpeed = 0x2000;
		}

		// split a rock, and move the object
		sourceObject.addSpeedFromVector(rockToSplit.mMotion, Fixed.FIXED_ONE_HALF);

		// POW!
		RockAudio.hitRock();
		rockToSplit.hit(mWindow.mRocksList);

		if (addScore)
		{
			mWindow.AddToScore(rockToSplit.mRockImage.getValueOfRock());
		}

		// only split if its large enough
		RockImage smallerImage = RockImage.getSmallerRockImage(rockToSplit.mRockImage);
		if (smallerImage != null)
		{
			RockObject newRock0, newRock1;
			int angle0, angle1;

			angle0 = vectorAngle - 23 + mRandom.nextInt(21) - 10;
			angle1 = vectorAngle + 23 + mRandom.nextInt(21) - 10;

			if ( ((angle0 / 90) * 90) == angle0)
			{
				angle0 = angle0 + 3;	// don't allow 90 degrees
			}
			if ( ((angle1 / 90) * 90) == angle1)
			{
				angle1 = angle1 + 3;	// don't allow 90 degrees
			}

			newRock0 = new RockObject (
							rockToSplit.mMotion.getX(), rockToSplit.mMotion.getY(),
							angle0, vectorSpeed,
							mWindow.mBounds,
							smallerImage);

			newRock1 = new RockObject (
							rockToSplit.mMotion.getX(), rockToSplit.mMotion.getY(),
							angle1, vectorSpeed,
							mWindow.mBounds,
							smallerImage);
			newRock1.mMotion.addSpeed(Fixed.FIXED_ONE_HALF);

			mWindow.mRocksList.add (newRock0);
			mWindow.mRocksList.add (newRock1);
		}
	}

	private void processFrame()
	{
		if (mWindow.mObjectsReady)
		{
			ListIterator	iterator;
			RockObject		rock;
			int				count;

			if (mWindow.mRocksList.size() == 0)
			{
				// no more rocks, time to create some news ones.
				RockAudio.goNextLevel(4);
				mWindow.mLevel++;
				mWindow.BuildWorld();
				mWindow.StartAnnouncer();
				mWindow.NewPlayfield();
			}

			// walk through missles and check for hit
			for (count = 0; count < mWindow.MAX_MISSILES; count++)
			{
				boolean missleHits;

				if (mWindow.mMissiles[count] != null)
				{
					mWindow.mMissiles[count].update(mWindow);

					missleHits = false;

					if (mWindow.IsAntagonistDead() == false)
					{
						if (mWindow.mAntagonist.collidesWith(mWindow.mMissiles[count]))
						{
							mWindow.AddToScore(kANTAGONIST_VALUE);
							mWindow.mAntagonist.hit();
							missleHits = true;
							mWindow.mMissiles[count] = null;
							break;
						}
					}

					if (mGOD == false)
					{
						// see if this missile can hurt our hero
						if (mWindow.mHero.collidesWith(mWindow.mMissiles[count]))
						{
							mWindow.mTimeBetweenShots = 0;
							mWindow.mLastShot = mGameFrame;
							mWindow.HitHero();
							missleHits = true;
							mWindow.mMissiles[count] = null;
							break;
						}
					}

					// now walk through all the rocks and check for missle hits and hero hits
					iterator = mWindow.mRocksList.listIterator(mWindow.mRockListIterator);
					rock = (RockObject) iterator.next();
					if (rock != null)
					{
						for (; rock != null ; rock = (RockObject) iterator.next())
						{
							if (rock.collidesWith(mWindow.mMissiles[count]))
							{
								boolean heroMissile = false;

								missleHits = true;

								if (mWindow.mMissiles[count].mOrigin == mWindow.mHero)
								{
									// only allow score for rock hits if its our hero
									heroMissile = true;
								}
								splitRock(mWindow.mMissiles[count].mMotion, rock, Fixed.fromInt(4), heroMissile);
								mWindow.mMissiles[count] = null;
								break;
							}
						}
					}
					if (!missleHits
						&& mWindow.mMissiles[count].mMotion.getCurrentFrame() ==
							(mWindow.mMissiles[count].mMotion.getMaxFrames() - 1) )
					{
							mWindow.mMissiles[count].mMotion = null;
							mWindow.mMissiles[count] = null;
					}
				}
			}

			// walk through active rocks
			iterator = mWindow.mRocksList.listIterator(mWindow.mRockListIterator);
			for (rock = (RockObject) iterator.next(); rock != null ; rock = (RockObject) iterator.next())
			{
				rock.update(mWindow);

				if (mWindow.mAntagonist != null)
				{
					if (mWindow.mAntagonist.collidesWith(rock))
					{
						if (mWindow.mAntagonist.sustainDamage() == false)
						{
							mWindow.mAntagonist.hit();
						}
						else
						{
							splitRock(mWindow.mAntagonist.mMotion, rock, Fixed.fromInt(2), false);
						}
					}
				}

				if (mGOD == false)
				{
					if (mWindow.mHero.collidesWith(rock))
					{
						mWindow.mTimeBetweenShots = 0;
						mWindow.mLastShot = mGameFrame;
						RockAudio.goNextLevel();
						if (mWindow.mHero.sustainDamage() == false)
						{
							mWindow.HitHero();
						}
						else
						{
							splitRock(mWindow.mHero.mMotion, rock, Fixed.fromInt(2), true);
						}
					}
				}
			}

			// handle auto fire
			if (mWindow.mAutoFire)
			{
				if (mWindow.mAutoFireTimer == 0)
				{
					mWindow.LaunchMissile();
					mWindow.mAutoFireTimer = 5;
				}
				else
				{
					mWindow.mAutoFireTimer--;
				}
			}

			// handle antagonist logic
			if (mWindow.IsAntagonistDead() == false)
			{

				if (mGOD == false)
				{
					Rect h = mWindow.mHero.getRect();
					Rect a = mWindow.mAntagonist.getRect();
					a.inset(4);

					if (h.intersects(a))
					{
						// hero and antagonist have collided; all is not well with the world.
						mWindow.mAntagonist.hit();
						mWindow.HitHero();
						RockAudio.goNextLevel();
					}
				}

				if (mWindow.mAntagonistAutoFireTimer == 0)
				{
					mWindow.AntagonistLaunchMissile();
					mWindow.mAntagonistAutoFireTimer = 25;
				}
				else
				{
					mWindow.mAntagonistAutoFireTimer--;
				}
			}

			mWindow.mTimeBetweenShots = mGameFrame - mWindow.mLastShot;
			if (mWindow.mTimeBetweenShots > 75)
			{
				if (mWindow.mAntagonistLastTimeOnScreen == 0)
				{
					if (mWindow.IsAntagonistDead() == true)
					{
						mWindow.NewAntagonist();
						mWindow.mTimeBetweenShots = 0;
					}
					else
					{
						mWindow.mAntagonistLastTimeOnScreen = mWindow.WAIT_TIME_ANTAGONIST;
					}
				}
				else
				{
					mWindow.mAntagonistLastTimeOnScreen--;
				}
			}

			// handle end game reset
			if (mWindow.mResetGameTimer != 0)
			{
				mWindow.mResetGameTimer--;
				if (mWindow.mResetGameTimer == 0)
				{
					restartGame();
				}
			}

			mWindow.Update();
			RockAudio.idleAudio();
		}
		mGameFrame++;
	}

	public static boolean isGamePaused()
	{
		return mPause;
	}

	public static void pauseGame()
	{
		if (mPause == false)
		{
			mPause = true;
			mWindow.pause();
			RockAudio.pause();
		}
	}

	public static void resumeGame()
	{
		if (mPause == true)
		{
			mPause = false;
			mWindow.resume();
			RockAudio.resume();
		}
	}

	//*	--------------------	ReceiveEvent

	public boolean
            receiveEvent(Event e)
	{
		switch (e.type)
		{
  			case kCmd_NewGame:
				newGame();
				resumeGame();
				break;
  			case kCmd_ResumeGame:
				resumeGame();
				break;
			case kCmd_SoundtrackOff:
			case kCmd_SoundtrackOn:
				RockAudio.toggleSoundtrack();
				break;
			case kCmd_Quit:
				returnToLauncher();
				break;
  			case kCmd_ExitGame:
				restartGame();
				resumeGame();
				break;
			case Event.EVENT_TIMER:
				if (mWindow.mGameState == mWindow.INTRO_SCREEN
					&& !Screen.isWindowActive(mWindow))
				{
					return true;
				}

				if (isGamePaused() == false)
				{
					if (!Screen.isWindowActive(mWindow))
					{
						pauseGame();
						return true;
					}
					else
					{
						processFrame();
					}
				}
//				else
				{
					// we still need to draw even though we're paused
					mWindow.invalidate();
				}
				return true;
		}
		return (super.receiveEvent(e));
	}

	public void
            quit()
	{
		suspend();
		unload();
	}

	//*	--------------------	Resume

	public void
            resume()
	{
		// 100 is 10 frames per second
		// 150 is 7.5 frames per second
		// 200 is 5 frames per second
		if (mFrameTimer == null)
			mFrameTimer = new Timer(100, true, this);

		mFrameTimer.start();
		mWindow.show();
		RockAudio.setup();
	}

	//*	--------------------	Suspend

	public void
            suspend()
	{
		if (mFrameTimer != null)
			mFrameTimer.stop();

		if (mWindow.mGameState != mWindow.INTRO_SCREEN)
		{
			pauseGame();
		}
		RockAudio.cleanup();
		updatePreviewScreen();
	}


	/* constants */
	static final int	MAX_LIVES			=	6;
	static final int	STARTING_LIVES		=	3;
	static final int	EXTRA_LIFE_AT		=	15000;
	static final int	ALT_AT				=	25000;

  	/* strings */
  	static final int	kCmd_ResumeGame		=	1;
  	static final int	kCmd_SoundtrackOn	=	2;
  	static final int	kCmd_SoundtrackOff	=	3;
  	static final int	kCmd_NewGame		=	4;
  	static final int	kCmd_ExitGame		=	5;
  	static final int	kCmd_Quit			=	6;

	// score values
	static final int kSMALL_ROCK_VALUE		= 150;
	static final int kMED_ROCK_VALUE		= 100;
	static final int kLARGE_ROCK_VALUE		= 50;
	static final int kANTAGONIST_VALUE		= 25;


	//*	--------------------------------	Class Variables
	static RocksStore	mDS;
	static boolean		mDEBUG				=	false;
	static boolean		mGOD				=	false;
	static boolean		mPause				=	false;
	static boolean		sIsColor;
	static RockWindow	mWindow;
	static Random		mRandom;
	static rocks		mMe;
	static int			mGameFrame;
	static Timer		mFrameTimer;

    // use Container to keep cross-OS working
	private Container	mSplashScreen;
	private StaticTextBox	mNumRemaining;
}


class RocksStore extends Listener
{
	RocksStore()
	{
		mSettings = new SettingsDB(kRocksDB, true);
		if (mSettings != null)
		{
			mSettings.setAutoSyncNotifyee(this);
		}
	}

	public int GetHighScore()
	{
		int value = mLocalHS;

		if (mSettings != null)
		{
			int value2 = 0;

			try
			{
				value2 = mSettings.getIntValue(kHighscoreTag);
			}
			catch (SettingsDBException e)
			{
			}
			if (value2 > mLocalHS)
			{
				value = value2;
				mLocalHS = value2;
			}
		}
		return value;
	}

	public void SetHighScore(int score)
	{
		if (mSettings != null)
		{
			mSettings.setIntValue(kHighscoreTag, score);
		}
		mLocalHS = score;
	}

	// settingsDB stuff
	static final String kRocksDB			= "Rocks";
	static final String kHighscoreTag		= "hs1";

	private SettingsDB	mSettings;
	private int			mLocalHS;
}



// EOF


